! $Id: grid.f90,v 1.1 2010/06/01 10:54:58 kg Exp $
!*******************************************************************************
MODULE grid_m                                                                   ! grid and other global parameters
  implicit none
  integer                             :: mpi_size=1, mpi_rank=0                 ! MPI size, our rank
  integer                             :: mpi_x=0, mpi_y=0, mpi_z=0              ! MPI coordinates
  integer                             :: mpi_mx=0, mpi_my=0, mpi_mz=0           ! MPI dimensions
  integer                             :: mx, my, mz, mvar=8                     ! dimensions, no. of variables
  integer                             :: nstars                                 ! number of stars
  integer, parameter                  :: txt_input=1, data_unit=2               ! file unit numbers
  logical                             :: master=.true.                          ! master process, does the i/o
  real                                :: t, dt, tend                            ! time, time step, end
  real                                :: sx, sy, sz                             ! global size
  real                                :: xmin, ymin, zmin                       ! global coordinate minima
  real                                :: xmax, ymax, zmax                       ! ditto maxima
  real                                :: xmin_mpi, ymin_mpi, zmin_mpi           ! local coordinate minima
  real                                :: xmax_mpi, ymax_mpi, zmax_mpi           ! ditto maxima
  real                                :: dx, dy, dz                             ! grid steps
  real                                :: starmass                               ! total stellar mass
  real, parameter                     :: pi=3.1415926536                        ! pi
  real, allocatable, dimension(:,:,:) :: &                                      ! grid arrays
    rho,lnr,pp,U,e,cA,divU,nu,lnu,ee,eta, &                                     ! scalar variables
    px,py,pz,Bx,By,Bz,Ux,Uy,Uz,Ex,Ey,Ez,jx,jy,jz,qx,qy,qz, &                    ! vector components
    Sxx,Syy,Szz,Sxy,Szx,Syz,Txx,Tyy,Tzz,Txy,Tzx,Tyz, &                          ! tensor components
    drdt,dedt,dpxdt,dpydt,dpzdt,dBxdt,dBydt,dBzdt, &                            ! time derivatives
    xdnrho,ydnrho,zdnrho,Uxc,Uyc,Uzc,Vxc,Vyc,Vzc,Vx,Vy,Vz, &                    ! staggered densities, centered velocities
    scr1,scr2,scr3,scr4,scr5,scr6, &                                            ! scratch arrays, look out for collisions!
    scratch                                                                     ! only to be used where no subroutines are called
  real, allocatable, dimension(:) :: x,y,z                                      ! local mesh
END MODULE

!*******************************************************************************
SUBROUTINE init_grid                                                            ! setup grid size and resolution
  USE grid_m
  implicit none
  integer i
  namelist /grid/ mx, my, mz, sx, sy, sz, mpi_mx, mpi_my, mpi_mz
!...............................................................................
  sx = 1.; sy = 1.; sz = 1.                                                     ! default = unit cube
  mpi_mx=1; mpi_my=0; mpi_mz=1                                                  ! default = no MPI domains
  rewind(txt_input); read(txt_input,grid)                                       ! grid namelist
  dx = sx/mx; dy = sy/my; dz = sz/mz                                            ! grid resolution
  xmin = -dx/2.; xmax=xmin+sx                                                   ! global x-minmax
  ymin = -dy/2.; ymax=ymin+sy                                                   ! global y-minmax
  zmin = -dz/2.; zmax=zmin+sz                                                   ! global z-minmax
  call init_mpi_grid                                                            ! this might modify the local grid
  if (master) write (*,grid)                                                    ! master process writes to log
  call allocate_arrays                                                          ! allocate (mx,my,mz) arrays
  x = (/ (i+mpi_x*mx, i=0,mx-1) /)*dx                                           ! linear x-scale
  y = (/ (i+mpi_y*my, i=0,my-1) /)*dy                                           ! linear y-scale
  z = (/ (i+mpi_z*mz, i=0,mz-1) /)*dz                                           ! linear z-scale
  xmin_mpi = (mpi_x*mx-0.5)*dx; xmax_mpi=((mpi_x+1)*mx-0.5)*dx                  ! local x-minmax
  ymin_mpi = (mpi_y*my-0.5)*dy; ymax_mpi=((mpi_y+1)*my-0.5)*dy                  ! local y-minmax
  zmin_mpi = (mpi_z*mz-0.5)*dz; zmax_mpi=((mpi_z+1)*mz-0.5)*dz                  ! local z-minmax
  print'(a,i4,1x,3i4,3f7.4)','init_grid:', &
    mpi_rank,mpi_x,mpi_y,mpi_z,xmin_mpi,ymin_mpi,zmin_mpi
  call init_stagger                                                             ! initialize staggered mesh operators
END SUBROUTINE

!*******************************************************************************
SUBROUTINE allocate_arrays
  USE grid_m
  implicit none
!...............................................................................
  allocate (rho(mx,my,mz), lnr(mx,my,mz), pp(mx,my,mz), U(mx,my,mz))            ! some of these could share memory ..
  allocate (cA(mx,my,mz), divU(mx,my,mz), nu(mx,my,mz), e(mx,my,mz))            ! .. but we keep things simple
  allocate (px(mx,my,mz), py(mx,my,mz), pz(mx,my,mz))
  allocate (qx(mx,my,mz), qy(mx,my,mz), qz(mx,my,mz))
  allocate (Bx(mx,my,mz), By(mx,my,mz), Bz(mx,my,mz))
  allocate (Ux(mx,my,mz), Uy(mx,my,mz), Uz(mx,my,mz))
  allocate (Vx(mx,my,mz), Vy(mx,my,mz), Vz(mx,my,mz))
  allocate (Ex(mx,my,mz), Ey(mx,my,mz), Ez(mx,my,mz))
  allocate (jx(mx,my,mz), jy(mx,my,mz), jz(mx,my,mz))
  allocate (Sxx(mx,my,mz), Syy(mx,my,mz), Szz(mx,my,mz))
  allocate (Sxy(mx,my,mz), Szx(mx,my,mz), Syz(mx,my,mz))
  allocate (Txx(mx,my,mz), Tyy(mx,my,mz), Tzz(mx,my,mz))
  allocate (Txy(mx,my,mz), Tzx(mx,my,mz), Tyz(mx,my,mz))
  allocate (drdt(mx,my,mz), dedt(mx,my,mz))
  allocate (dpxdt(mx,my,mz), dpydt(mx,my,mz), dpzdt(mx,my,mz))
  allocate (dBxdt(mx,my,mz), dBydt(mx,my,mz), dBzdt(mx,my,mz))
  allocate (eta(mx,my,mz), lnu(mx,my,mz), ee(mx,my,mz))
  allocate (xdnrho(mx,my,mz), ydnrho(mx,my,mz), zdnrho(mx,my,mz))
  allocate (Uxc(mx,my,mz), Uyc(mx,my,mz), Uzc(mx,my,mz))
  allocate (Vxc(mx,my,mz), Vyc(mx,my,mz), Vzc(mx,my,mz))
  allocate (scr1(mx,my,mz), scr2(mx,my,mz), scr3(mx,my,mz))                     ! for local scratch, shared
  allocate (scr4(mx,my,mz), scr5(mx,my,mz), scr6(mx,my,mz))                     ! for local scratch, shared
  allocate (scratch(mx,my,mz))                                                  ! used only where no subroutines are called
  allocate (x(mx), y(my), z(mz))
END SUBROUTINE

!*******************************************************************************
SUBROUTINE deallocate_arrays                                                    ! deallocate to avoid nose from g95
  USE grid_m
  implicit none
!.......................................................................
  deallocate (rho, lnr, pp, U)
  deallocate (cA, divU, nu, e)
  deallocate (px, py, pz)
  deallocate (qx, qy, qz)
  deallocate (Bx, By, Bz)
  deallocate (Ux, Uy, Uz)
  deallocate (Vx, Vy, Vz)
  deallocate (Ex, Ey, Ez)
  deallocate (jx, jy, jz)
  deallocate (Sxx, Syy, Szz)
  deallocate (Sxy, Szx, Syz)
  deallocate (Txx, Tyy, Tzz)
  deallocate (Txy, Tzx, Tyz)
  deallocate (drdt, dedt, ee)
  deallocate (dpxdt, dpydt, dpzdt)
  deallocate (dBxdt, dBydt, dBzdt)
  deallocate (eta, lnu)
  deallocate (xdnrho, ydnrho, zdnrho)
  deallocate (Uxc, Uyc, Uzc)
  deallocate (Vxc, Vyc, Vzc)
  deallocate (scr1, scr2, scr3)
  deallocate (scr4, scr5, scr6)
  deallocate (scratch)
  deallocate (x, y, z)
END subroutine
