! $Id: mhd.f90,v 1.1 2010/06/01 10:54:58 kg Exp $
!*******************************************************************************
MODULE mhd_m
  real    :: nu0, nu1, nu2, nu3, nur, nue, nuB, nuS, gamma
END MODULE

!*******************************************************************************
SUBROUTINE init_mhd
  USE mhd_m
  USE grid_m, only: txt_input, master
  implicit none
  namelist /mhd/ nu0, nu1, nu2, nu3, nur, nue, nuB, nuS, gamma
!...............................................................................
  gamma = 5./3.
  nu0=0.007; nu1=0.050; nu2=1.2; nu3=0.; nur=.1; nue=1.; nuB=1.0; nuS=1.
  rewind(txt_input); read (txt_input,mhd); if (master) write(*,mhd)
END SUBROUTINE

!*******************************************************************************
SUBROUTINE pde
  USE grid_m
  USE stagger_m
  USE mhd_m
  implicit none

!-----------------------------------------------------------------------
!  Density interpolations
!-----------------------------------------------------------------------
  lnr = alog(rho)
  xdnrho = exp(xdn(lnr))
  ydnrho = exp(ydn(lnr))
  zdnrho = exp(zdn(lnr))

!-----------------------------------------------------------------------
!  Velocities
!-----------------------------------------------------------------------
  Ux = px/xdnrho; Uxc = xup(Ux)
  Uy = py/ydnrho; Uyc = yup(Uy)
  Uz = pz/zdnrho; Uzc = zup(Uz)
  U  = sqrt(Uxc**2 + Uyc**2  + Uzc**2)
  
!-----------------------------------------------------------------------
!  Equation of state 
!-----------------------------------------------------------------------
  if (gamma==1) then
    pp = rho
  else
    pp = (gamma-1.)*e
  end if
  cA = sqrt((xup(Bx)**2 + yup(By)**2  + zup(Bz)**2 + pp*gamma)/rho)

!-----------------------------------------------------------------------
!  Numerical diffusion and resistivity, Richtmyer & Morton type
!-----------------------------------------------------------------------
  divU = ddxup(Ux) + ddyup(Uy) + ddzup(Uz)
  nu  = nu0*dx + nu1*dx*(U+cA) + nu2*dx**2*smooth3(max3(max(-divU,0.0)))
  if (nu3 > 0) nu = nu + &
   nu3*dx**3*(xup1(ddxdn(lnr)**2)+yup1(ddydn(lnr)**2)+zup1(ddzdn(lnr)**2))
  eta = nuB*nu

!-----------------------------------------------------------------------
!  Mass conservation, cell centered
!-----------------------------------------------------------------------
  qx = px - nur*exp(xdn(alog(nu)))*ddxdn(rho)
  qy = py - nur*exp(ydn(alog(nu)))*ddydn(rho)
  qz = pz - nur*exp(zdn(alog(nu)))*ddzdn(rho)
  Vx = qx/xdnrho; Vxc = xup(Vx)
  Vy = qy/ydnrho; Vyc = yup(Vy)
  Vz = qz/zdnrho; Vzc = zup(Vz)
  drdt = drdt - ddxup(qx) - ddyup(qy) - ddzup(qz)

!-----------------------------------------------------------------------
!  Viscous stress, diagonal and off-diagonal elements
!-----------------------------------------------------------------------
  Sxx = ddxup(Ux)
  Syy = ddyup(Uy)
  Szz = ddzup(Uz)
  Txx = - nuS*nu*rho*Sxx - (1.-nuS)*nu*rho*divU
  Tyy = - nuS*nu*rho*Syy - (1.-nuS)*nu*rho*divU
  Tzz = - nuS*nu*rho*Szz - (1.-nuS)*nu*rho*divU
  Sxy = 0.5*(ddxdn(Uy)+ddydn(Ux))
  Syz = 0.5*(ddydn(Uz)+ddzdn(Uy))
  Szx = 0.5*(ddzdn(Ux)+ddxdn(Uz))
  lnu = alog(nu*rho)
  Txy = - nuS*exp(xdn(ydn(lnu)))*Sxy
  Tyz = - nuS*exp(ydn(zdn(lnu)))*Syz
  Tzx = - nuS*exp(zdn(xdn(lnu)))*Szx

!-----------------------------------------------------------------------
!  Electric current j = curl(B), edge centered
!-----------------------------------------------------------------------
  jx = ddydn(Bz) - ddzdn(By)
  jy = ddzdn(Bx) - ddxdn(Bz)
  jz = ddxdn(By) - ddydn(Bx)

!-----------------------------------------------------------------------
!  Lorentz force
!-----------------------------------------------------------------------
  dpxdt = dpxdt + zup(jy*xdn(Bz)) - yup(jz*xdn(By))
  dpydt = dpydt + xup(jz*ydn(Bx)) - zup(jx*ydn(Bz))
  dpzdt = dpzdt + yup(jx*zdn(By)) - xup(jy*zdn(Bx))

!-----------------------------------------------------------------------
!  Electric field   E = eta j - (u x B), edge centered
!-----------------------------------------------------------------------
  Ex = ydn(zdn(eta))*jx
  Ey = zdn(xdn(eta))*jy
  Ez = xdn(ydn(eta))*jz
  if (gamma .ne. 1.) then
    dedt = dedt + zup(yup(jx*Ex) + xup(jy*Ey)) + xup(yup(jz*Ez))
  end if
  Ex = Ex - zdn(Vy)*ydn(Bz) + ydn(Vz)*zdn(By)
  Ey = Ey - xdn(Vz)*zdn(Bx) + zdn(Vx)*xdn(Bz)
  Ez = Ez - ydn(Vx)*xdn(By) + xdn(Vy)*ydn(Bx)

!-----------------------------------------------------------------------
!  Magnetic field's time derivative, dBdt = - curl(E), face centered
!-----------------------------------------------------------------------
  dBxdt = dBxdt + ddzup(Ey) - ddyup(Ez)
  dBydt = dBydt + ddxup(Ez) - ddzup(Ex)
  dBzdt = dBzdt + ddyup(Ex) - ddxup(Ey)

!-----------------------------------------------------------------------
!  Energy eq, with P*dV work, difffusion, kinetic and Joule dissipation
!-----------------------------------------------------------------------
  if (gamma .ne. 1) then
    ee = e/rho
    dedt = dedt - pp*divU &
         - ddxup (xdn(ee)*px - nue*exp(xdn(lnu))*ddxdn(ee)) &
         - ddyup (ydn(ee)*py - nue*exp(ydn(lnu))*ddydn(ee)) &
         - ddzup (zdn(ee)*pz - nue*exp(zdn(lnu))*ddzdn(ee)) &
         - (Txx*Sxx + Tyy*Syy + Tzz*Szz) &
         - 2.*(xup1(yup1(Txy*Sxy) + zup1(Tzx*Szx)) + yup1(zup1(Tyz*Syz)))
  end if
!-----------------------------------------------------------------------
!  Pressure and Reynolds stress
!-----------------------------------------------------------------------
  Txx = Txx + pp + rho*Uxc*Vxc
  Tyy = Tyy + pp + rho*Uyc*Vyc
  Tzz = Tzz + pp + rho*Uzc*Vzc
  Txy = Txy + exp(xdn(ydn(lnr)))*ydn(Ux)*xdn(Vy)
  Tzx = Tzx + exp(xdn(zdn(lnr)))*zdn(Ux)*xdn(Vz)
  Tyz = Tyz + exp(ydn(zdn(lnr)))*zdn(Uy)*ydn(Vz)

!-----------------------------------------------------------------------
!  Equations of motion, face centred
!-----------------------------------------------------------------------
  dpxdt = dpxdt - ddxdn(Txx) - ddyup(Txy) - ddzup(Tzx)
  dpydt = dpydt - ddydn(Tyy) - ddzup(Tyz) - ddxup(Txy)
  dpzdt = dpzdt - ddzdn(Tzz) - ddxup(Tzx) - ddyup(Tyz)

  END
