! $Id: timestep.f90,v 1.2 2010/06/03 12:53:03 kg Exp $
!*******************************************************************************
MODULE time_m
  USE grid_m
  real    :: Cdt                                                                ! Courant number
  integer :: isubstep
  real    :: alpha(3), beta(3)
  data alpha /     0., -0.641874, -1.31021/                                     ! "magic" numbers used by ..
  data beta  /0.46173,  0.924087, 0.390614/                                     ! .. the time stepping method
END MODULE

!*******************************************************************************
SUBROUTINE init_timestep
  USE time_m
  USE grid_m, only: txt_input, master
  USE io_m, only: nprint, it
  implicit none
  namelist /timestp/ Cdt, dt, nprint
!...............................................................................
  Cdt   = 0.4
  t     = 0.
  dt    = 0.01
  it    = 0
  nprint= max(1,ceiling(1./(8e-6*mx*my*mz)))
  rewind(txt_input); read (txt_input,timestp); if (master) write (*,timestp) 
END SUBROUTINE

!*******************************************************************************
SUBROUTINE timestep
!
!  3rd order, 2-N storage Runge-Kutta.  Advances the eight MHD variables
!  forward in time, given time derivatives computed in the subroutine pde.
!  See Williamson 1980, Journal of Computational Physics, Vol. 35, p.48
!
  USE time_m
  USE grid_m
  USE io_m, only: it
  implicit none
!...............................................................................
  do isubstep=1,3                                                               ! three Runge-Kutta sub-steps
    drdt  = alpha(isubstep)*drdt
    dpxdt = alpha(isubstep)*dpxdt
    dpydt = alpha(isubstep)*dpydt
    dpzdt = alpha(isubstep)*dpzdt
    dedt  = alpha(isubstep)*dedt
    dBxdt = alpha(isubstep)*dBxdt
    dBydt = alpha(isubstep)*dBydt
    dBzdt = alpha(isubstep)*dBzdt

    call pde                                                                    ! get the dfdt's (mhd.f90)
    if (isubstep==1) call Courant                                               ! set dt in the first sub-step
    
    rho = rho + (dt*beta(isubstep))*drdt
    px  = px  + (dt*beta(isubstep))*dpxdt
    py  = py  + (dt*beta(isubstep))*dpydt
    pz  = pz  + (dt*beta(isubstep))*dpzdt
    e   = e   + (dt*beta(isubstep))*dedt
    Bx  = Bx  + (dt*beta(isubstep))*dBxdt
    By  = By  + (dt*beta(isubstep))*dBydt
    Bz  = Bz  + (dt*beta(isubstep))*dBzdt
  end do

  t  = t + dt                                                                   ! update the time
  it = it + 1                                                                   ! update the step count
END

!*******************************************************************************
SUBROUTINE Courant                                                              ! Courant condition
  USE grid_m
  USE time_m
  USE io_m, only: tsnap, nsnap, it, nprint
  USE stagger_m
  implicit none
  real Cu, Cv, Cn, Cr, Ce, fmaxval, wallclock, wc1, rem, B2, Urms, faver, Umax, rhomax
  real, save:: wc0=0., wc2=0., t0
  integer, save:: iprint=0, nheader=25
  character(len=1):: unit
!...............................................................................
  Cu = fmaxval(U+cA)*dt/dx                                                      ! wave speed
  Cv = 6.2*3.*fmaxval(nu )/2.*dt/dx**2*0.4/1.2                                  ! viscous diffusion speed
  Cn = 6.2*3.*fmaxval(eta)/2.*dt/dx**2*0.4/1.2                                  ! resistive diffusion speed
  Cr = 2.*dt*fmaxval(abs(drdt/rho))                                             ! density rate of change
  Ce = 2.*dt*fmaxval(abs(dedt/e))                                               ! energy rate of change
  dt = dt*Cdt/max(Cu,Cv,Cn,Cr,Ce)                                               ! set new time step

  wc1 = wallclock()                                                             ! wall clock
  if (wc0==0.) then                                                             ! save initial times
    t0 = t
    wc0 = wc1 
    wc2 = wc1
  end if
  rem = (wc1-wc0)*(tsnap*nsnap-t)/max(t-t0,dt)                                  ! remaining time
  unit = 's'                                                                    ! in seconds
  if (rem > 100.) then
    rem = rem/60.                                                               ! in minutes
    unit = 'm'
    if (rem > 100.) then
      rem = rem/60.                                                             ! in hours
      unit = 'h'
      if (rem >  48.) then
        rem = rem/24.                                                           ! in days
        unit = 'd'
      end if
    end if
  end if

  if (mod(it,nprint)==0) then
    Urms = sqrt(faver(U**2))
    Umax = fmaxval(U)
    rhomax = fmaxval(rho)
    B2 = faver(sqrt(xup(Bx)**2+yup(By)**2+zup(Bz)**2))*0.5
    if (master) then
      if (mod(iprint,nheader)==0) print'(/a)', &
'   it    time    dt          <------ Courant numbers ------->   Benergy    Urms   Umax  micros  remains'
!   26    0.0197  0.000441    0.41   0.13   0.13   0.00   0.00    5.00      1.85   3.72   7.35    36.0m
      iprint = iprint+1
      print'(i6,f9.4,f10.6,1x,5f7.2,2x,f8.4,f8.2,3f7.2,a3,f7.3)', &
        it,t,dt,Cu,Cv,Cn,Cr,Ce, &
        B2,Urms,Umax,(wc1-wc2)*1e6/(mx*my*mz),rem,unit
    end if

  endif
  wc2 = wc1                                                                     ! remember wall clock
END SUBROUTINE
!*******************************************************************************
